# -*- coding: utf-8 -*-

# Standard Library
import datetime

# Vendor Library
from django.db.models import Q

# Project Library
from kinder.core.declaration_status.models import DSS
from kinder.core.dict.models import (
    HealthNeed, GroupTypeEnumerate, PrivilegeType)
from kinder.core.direct.models import DRS, Direct
from kinder.core.group.models import Pupil, GroupStatusEnum, Group
from ..utils import DateMixin
from gisdo.algorithm.constants import (
    get_all_day_work_types, get_short_day_types)


def get_1_filter(context):
    """Фильтр для показателя 1
    - все заявления, кроме заявлений
      в статусе "Архивная", "Отказано в услуге",
      "Не явился", "Проверка данных ЗАГС"
    - Дата подачи >= 01.01.2015
    - В заявках учитываем только первый приоритетный ДОУ
    - Не разбивать по возрастам.
    - Показатель считается по всем видам ДОУ
    :return: Q
    """

    date_border = datetime.date.today()

    return (
        ~Q(status__code__in=[
            DSS.ARCHIVE, DSS.DUL_CONFIRMATING,
            DSS.TUTOR_CONFIRMATING, DSS.MED_CONFIRMATING,
            DSS.DIRECTED, DSS.NOT_ATTENDED]) &
        Q(date__gte=datetime.date(date_border.year, 1, 1))
    )


def get_4_filter(context):
    """Фильтр показателя 4
    - все заявки в статусах
      "Зарегистрировано", "Подтверждение льгот", "Желает изменить ДОУ"
    - заявки в других статусах,
      у которых есть привязанные направления в статусах
      "Заключение договора", "Подтверждено", "Направлен в ДОУ",
      "Предложено системой".
    - дата подачи заявления = [01.09.2014; 31.08.2015] или текущий учебный год
    - желаемая дата зачисления <=31.08.2015 или текущий учебный год
    - дата расчета возраста = текущая дата
    - В заявках учитываем только первый приоритетный ДОУ
    - Показатель считается по всем видам ДОУ
    - Разбивка по возрастам: таблица 16

    :return: Q
    """

    d = DateMixin._get_next_calendar_year()
    date_begin, date_end = DateMixin.get_next_learn_year()

    return (
        Q(
            queue_with_want_change_dou(context)
        ) &
        Q(date__gte=datetime.date(date_begin.year, 9, 1),
          date__lte=date_end) &
        Q(desired_date__lte=datetime.date(d.year, 8, 31))
    )


def get_41_filter(context):
    """Фильтр для показателя 4.1
    - учитывать заявления в статусе "Зарегистрировано"
       "Желает изменить ДОУ" и "Подтверждение льгот"
    - заявки в других статусах, у которых есть привязанные
      направления в статусах "Заключение договора", "Подтверждено",
      "Направлен в ДОУ", "Предложено системой".
    - дата подачи заявления = [01.09.2014; 31.08.2015] или текущий учебный год
    - желаемая дата зачисления <=31.08.2015 или текущий учебный год
    - учитывать первый приоритетный ДОУ
    - в заявлении в поле "ОВЗ" указано значение,
      соответствующее в справочнике "Ограниченные возможности здоровья"
      тип группы = компенсирующий, исключаются значения "Нет"
    - Показатель считается по всем видам ДОУ
    - дата расчета возраста = текущая дата
    - разбивка по возрастам - 16 категорий

    :return: Q
    """

    d = DateMixin._get_next_calendar_year()
    date_begin, date_end = DateMixin.get_next_learn_year()

    return (
        Q(
            queue_with_want_change_dou(context)
        ) &
        Q(desired_date__lte=datetime.date(d.year, 8, 31)) &
        Q(children__health_need__group_type__code=GroupTypeEnumerate.COMP) &
        with_health_need_filter() &
        Q(date__gte=datetime.date(date_begin.year, 9, 1),
          date__lte=date_end)
    )


def get_42_filter(context):
    """Фильтр для показателя 4.2
    - все заявки в статусе "Зарегистрировано",
      "Подтверждение льгот", "Желает изменить ДОУ"
    - заявки в других статусах, у которых есть
      привязанные направления в статусах "Заключение договора",
      "Подтверждено", "Направлен в ДОУ", "Предложено системой".
    - дата желаемого зачисления интервал <= 31.08.2015
    - дата расчета возраста = текущая дата
    - Показатель считается по всем видам ДОУ
    - накладываем фильтр по наличию
      оздоровительной потребности ( можно вынести из справочника
      "Ограниченные возможности здоровья" тип группы = оздоровительный)
    - разбивка по возрастам с периодичностью в полгода - 16 категорий

    :return: Q
    """

    date_border = DateMixin._get_next_calendar_year()
    date_begin, date_end = DateMixin.get_next_learn_year()

    return (
        Q(
            queue_with_want_change_dou(context)
        ) &
        Q(desired_date__lte=datetime.date(date_border.year, 8, 31)) &
        Q(children__health_need__group_type__code=GroupTypeEnumerate.HEALTH) &
        Q(date__gte=datetime.date(date_begin.year, 9, 1),
          date__lte=date_end)
    )


def get_5_filter(context):
    """Фильтр для показателя 5
    - заявления в статусах , "Желает изменить ДОУ"
    - заявки в других статусах, у которых есть привязанные
      направления в статусах "Заключение договора",
      "Подтверждено", "Направлен в ДОУ", "Предложено системой".
    - дата подачи заявления = [01.09.2014; 31.08.2015] или текущий учебный год
    - желаемая дата зачисления <=31.08.2015 или текущий учебный год
    - Показатель считается по всем видам ДОУ
    - расчет возраста на текущую дату
    - по первому приоритетному ДОУ
    - разбивка по возрастной категории - 16

    :return: Q
    """

    d = DateMixin._get_next_calendar_year()
    date_begin, date_end = DateMixin.get_next_learn_year()

    return (
        Q(status__code=DSS.WANT_CHANGE_DOU) &
        Q(date__gte=datetime.date(date_begin.year, 9, 1)) &
        Q(date__lte=datetime.date(date_end.year, 8, 31)) &
        Q(desired_date__lte=datetime.date(d.year, 8, 31))
    )


def get_7_filter(context):
    """Фильтр для показателя 7
    - заявки в статусах "Зарегистрировано" и "Подтверждение льгот".
    - заявки в других статусах, у которых есть привязанные
      направления в статусах "Заключение договора", "Подтверждено",
      "Направлен в ДОУ", "Предложено системой".
    - Дата желаемого зачисления текущий учебный год - >= 02.09.2015
      (т.е. все заявки позже этой даты (включительно)).
    - Дата расчета возраста: на 01.09.2015 (текущего календарного года.)
    - возрастная категория 16
    - по первому приоритетному ДОУ
    - Тип ДОУ = любой ДОУ
    :return: Q
    """

    return (
        Q(
            queue_with_direct(context) &
            ~Q(status__code=DSS.WANT_CHANGE_DOU)
        ) &
        Q(
            Q(after_desired_date_in_current_learn_year()) |
            Q(
                Q(desired_date__isnull=True) &
                Q(children__date_of_birth__gte=
                  DateMixin._get_current_calendar_year())
            )
        )
    )


def get_71_filter(context):
    """Фильтр для показателя 7.1
    - Учитываются заявки в статусах "Зарегистрировано" и "Подтверждение льгот"
    - заявки в других статусах, у которых есть привязанные
      направления в статусах "Заключение договора", "Подтверждено",
      "Направлен в ДОУ", "Предложено системой".
    - Дата желаемого зачисления текущий учебный год >= с 02.09.2015
      (т.е. все заявки позже этой даты (включительно)).
    - Дата расчета возраста ребенка: на текущую дату.
    - возрастная категория 16
    - Тип ДОУ = любой ДОУ
    - по первому приоритетному ДОУ
    :return: Q
    """

    return (
        Q(
            queue_with_direct(context) &
            ~Q(status__code=DSS.WANT_CHANGE_DOU)
        ) &
        Q(
            Q(after_desired_date_in_current_learn_year()) |
            Q(
                Q(desired_date__isnull=True) &
                Q(children__date_of_birth__gte=
                  DateMixin._get_current_calendar_year())
            )
        )
    )


def get_72_filter(context):
    """Фильтр для показателя 7.2
    - все заявки в статусе "Зарегистрировано", "Подтверждение льгот"
    - заявки в других статусах, у которых есть привязанные
      направления в статусах "Заключение договора", "Подтверждено",
      "Направлен в ДОУ", "Предложено системой".
    - тип ДОУ в заявке = любой
    - дата дата желаемого зачисления интервал [2.09.2015; 31.08.2016]
      период - после текущего учебного года
    - дата расчета возраста = текущая дата
    - возрастная категория 16
    - по первому приоритетному ДОУ
    :return: Q
    """

    return (
        Q(
            queue_with_direct(context) &
            ~Q(status__code=DSS.WANT_CHANGE_DOU)
        ) &
        Q(desired_date_interval_current_learn_year())
    )


def get_73_filter(context):
    """Фильтр для показателя 7.3
    :return: Q
    """

    next_calendar_year = DateMixin._get_next_calendar_year()

    return (
        Q(
            queue_with_direct(context) &
            ~Q(status__code=DSS.WANT_CHANGE_DOU)
        ) &
        Q(desired_date__gt=next_calendar_year)
    )


def get_74_filter(context):
    """Фильтр для показателя 7.4
    :return: Q
    """

    return Q(
        ~Q(status__code=DSS.WANT_CHANGE_DOU) &
        Q(direct__status__code__in=[DRS.REGISTER, DRS.DOGOVOR]) &
        ~Q(direct__group__status=GroupStatusEnum.ARCHIVE),
        desired_date_interval_current_learn_year()
    )


def get_75_filter(context):
    """Фильтр для показателей 7.5 и 7.6.
    :return: Q
    """

    return Q(
        Q(
            status_registration_or_privilege_confirmation() |
            Q(
                ~Q(status__code=DSS.WANT_CHANGE_DOU) &
                Q(direct__status__code__in=[DRS.NEW, DRS.CONFIRM])
            )
        ) &
        Q(
            declarationunit__ord=1,
            declarationunit__unit__id=context.unit.id
        ) & Q(
            before_desired_date_in_next_learn_year() |
            Q(desired_date__isnull=True)
        )
    )


def get_8_filter(context):
    from_date = datetime.datetime.now()
    from_date = from_date.replace(year=from_date.year - 2)
    return Q(
        Q(direct__isnull=False) &
        Q(direct__status__code__in=(
            DRS.ACCEPT,
            DRS.REGISTER,
            DRS.DOGOVOR,
            DRS.REJECT,
            DRS.NEED_CHANGE,
            DRS.REFUSED,
        )) &
        Q(direct__date__isnull=False) &
        Q(direct__date__gte=from_date) &
        Q(desired_date__isnull=False)
    )


def get_81_filter(context):
    return Q(
        get_8_filter(context) &
        Q(best_privilege__isnull=False)
    )


def get_82_filter(context):
    return Q(
        get_8_filter(context) &
        Q(best_privilege__isnull=True)
    )


def get_83_filter(context):
    return Q(
        get_8_filter(context) & with_health_need_filter()
    )


def get_9_filter(context):
    """Фильтр для показателя 9
    :param context:
    :return: Q
    """

    return (
        Q(
            queue_register_with_direct(context) &
            ~Q(status__code=DSS.WANT_CHANGE_DOU)
        ) &
        Q(best_privilege__isnull=False) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_10_filter(context):
    """Фильтр для показателя 10
    :return: Q
    """

    return (
        Q(
            queue_register_with_direct(context) &
            ~Q(status__code=DSS.WANT_CHANGE_DOU)
        ) &
        Q(best_privilege__type_id=PrivilegeType.FED) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_11_filter(context):
    """Фильтр для показателя 11
    :return: Q
    """

    return (
        Q(
            queue_register_with_direct(context) &
            ~Q(status__code=DSS.WANT_CHANGE_DOU)
        ) &
        Q(best_privilege__type_id__in=[
            PrivilegeType.MUN, PrivilegeType.REG]) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_12_filter(context):
    """Фильтр для показателя 12
    :return: Q
    """

    return (
        Q(
            queue_with_want_change_dou(context) &
            ~Q(direct__status__code__in=[DRS.DOGOVOR, DRS.REGISTER])
        ) &
        health_need_filter(range(1, 14)) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_121_filter(context):
    """Фильтр для показателя 12.1
    :return: Q
    """

    return (
        Q(direct__status__code__in=[DRS.REGISTER, DRS.DOGOVOR]) & ~Q(
            direct__group__status=GroupStatusEnum.ARCHIVE) &
        health_need_filter(range(1, 14)) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_13_filter(context):
    """Фильтр для показателя 13
    :return:
    """

    return (
        Q(
            Q(status__code=DSS.REGISTERED) |
            Q(
                Q(direct__status__code__in=[
                    DRS.CONFIRM, DRS.NEW]) & ~Q(
                    direct__group__status=GroupStatusEnum.ARCHIVE
                )
            )
        ) & ~Q(
            status__code=DSS.WANT_CHANGE_DOU) &
        health_need_filter(range(1, 14)) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(consent_dev_group=True) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_131_filter(context):
    """Фильтр для показателей 13.1
    :return: Q
    """

    return (
        Q(direct__status__code__in=[DRS.REGISTER, DRS.DOGOVOR]) & ~Q(
            direct__group__status=GroupStatusEnum.ARCHIVE) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        without_pupil(context) &
        health_need_filter(range(1, 14)) &
        Q(consent_dev_group=True) &
        Q(direct__group__type__code=GroupTypeEnumerate.DEV) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_14_filter(context):
    """Фильтр для показателя 14
    :return: Q
    """

    return (
        Q(
            Q(
                Q(status__code=DSS.REGISTERED) |
                Q(status__code=DSS.PRIV_CONFIRMATING) |
                Q(status__code=DSS.WANT_CHANGE_DOU)
            ) |
            Q(
                Q(direct__status__code__in=[DRS.CONFIRM, DRS.NEW]) & ~Q(
                    direct__group__status=GroupStatusEnum.ARCHIVE)
            )
        ) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        health_need_filter([14, 15, 16]) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_15_filter(context):
    """Фильтр для показателя 15
    :return: Q
    """

    return (
        Q(
            queue_with_direct_status_confirm_new(context) &
            ~Q(status__code=DSS.WANT_CHANGE_DOU)
        ) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(consent_care_group=True) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_16_filter(context):
    """Фильтр для показателя 16
    :return: Q
    """

    return (
        Q(
            queue_with_direct_status_confirm_new(context) &
            ~Q(status__code=DSS.WANT_CHANGE_DOU)
        ) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(work_type__code__in=get_all_day_work_types()) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_17_filter(context):
    """Фильтр для показателя 17
    :return: Q
    """

    return (
        Q(
            queue_with_direct_status_confirm_new(context) &
            ~Q(status__code=DSS.WANT_CHANGE_DOU)
        ) &
        Q(work_type__code__in=get_short_day_types()) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_171_filter(context):
    """Фильтр для показателя 17.1
    :return: Q
    """

    return (
        Q(
            queue_with_direct_status_confirm_new(context) &
            ~Q(status__code=DSS.WANT_CHANGE_DOU)
        ) &
        Q(consent_short_time_group=True) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_18_filter(context):
    """Фильтр для показателей 18
    :return: Q
    """

    return (
        Q(
            status_registration_or_privilege_confirmation() |
            Q(
                ~Q(status__code=DSS.WANT_CHANGE_DOU) &
                Q(direct__status__code__in=[DRS.NEW, DRS.CONFIRM]) & ~Q(
                    direct__group__status=GroupStatusEnum.ARCHIVE)
            )
        ) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_181_filter(context):
    """Фильтр для показателей 18.1
    :return: Q
    """

    return (
        Q(
            status_registration_or_privilege_confirmation() |
            Q(
                ~Q(status__code=DSS.WANT_CHANGE_DOU) &
                Q(direct__status__code__in=[DRS.NEW, DRS.CONFIRM]) & ~Q(
                    direct__group__status=GroupStatusEnum.ARCHIVE)
            )
        ) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_182_filter(context):
    """Фильтр для показателей 18.2
    :return: Q
    """

    return (
        Q(
            status_registration_or_privilege_confirmation() |
            Q(
                ~Q(status__code=DSS.WANT_CHANGE_DOU) &
                Q(direct__status__code__in=[DRS.NEW, DRS.CONFIRM]) & ~Q(
                    direct__group__status=GroupStatusEnum.ARCHIVE)
            )
        ) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_183_filter(context):
    """Фильтр для показателя 18.3
    :return: Q
    """

    return (
        ~Q(status__code=DSS.WANT_CHANGE_DOU) &
        Q(direct__status__code__in=[DRS.REGISTER, DRS.DOGOVOR]) & ~Q(
            direct__group__status=GroupStatusEnum.ARCHIVE) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_20_filter(context):
    """Фильтр для показателя 20
    - заявка в статусе "Желает изменить ДОУ"
    - есть связанное зачисление в другой детский сад, не учитываем
    зачисления с признаком Временно отчислен
    """
    children_ids = Pupil.objects.filter(
        grup__status=GroupStatusEnum.FACT,
        temporary_deduct=False
    ).exclude(
        grup__unit_id=context.unit.id,
    ).values_list('children', flat=True).distinct()

    return (
        Q(
            Q(status__code=DSS.WANT_CHANGE_DOU) |
            Q(
                Q(direct__status__code__in=[
                    DRS.CONFIRM, DRS.NEW]) & ~Q(
                        direct__group__status=GroupStatusEnum.ARCHIVE
                ) & Q(children_id__in=children_ids)
            )
        ) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_201_filter(context):
    """Фильтр для показателя 20.1
    - У ребенка есть зачисление в другой детсад
    - заявка в статусе Желает изменить ДОУ
    - Или направление в статусах Заключение договора, Направлен в ДОУ
    """

    children_ids = Pupil.objects.filter(
        grup__status=GroupStatusEnum.FACT,
        temporary_deduct=False
    ).exclude(
        grup__unit_id=context.unit.id,
    ).values_list('children', flat=True).distinct()

    allowed_direct_statuses = (DRS.REGISTER, DRS.DOGOVOR)
    direct_filter = Q(direct__in=[])
    for group in Group.objects.filter(
            unit=context.unit).filter(status=GroupStatusEnum.FACT):
        direct_in_group = Direct.objects.filter(
            Q(group=group) &
            Q(status__code__in=allowed_direct_statuses)
        ).values_list('id', flat=True)

        bounded_plan_groups = Group.objects.filter(
            Q(room_id__isnull=False, room_id=group.room_id),
            status=GroupStatusEnum.PLAN)

        direct_count_in_bounded_group = Direct.objects.filter(
            Q(group__in=bounded_plan_groups) &
            Q(status__code__in=[DRS.REGISTER, DRS.DOGOVOR])
        ).exclude(
            declaration__children_id__in=Direct.objects.filter(
                Q(group__unit=group.unit) &
                Q(group__status=GroupStatusEnum.FACT) &
                Q(status__code__in=[DRS.REGISTER, DRS.DOGOVOR])
            ).values_list("declaration__children_id", flat=True)
        ).values_list('id', flat=True)
        direct_filter |= (
            Q(direct__in=direct_in_group) | Q(
                direct__in=direct_count_in_bounded_group)
        )

    return (
        direct_filter &
        Q(children_id__in=children_ids) &
        Q(direct__group__unit_id=context.unit.id)
    )


def get_202_filter(context):
    """Фильтр для показателя 20.2
    - заявка в статусе "Желает изменить ДОУ"
    - есть связанное зачисление в другой детский сад, не учитываем
    зачисления с признаком Временно отчислен
    - с ОВЗ, поле заполнено, кроме пусто и нет
    """

    return (
        Q(status__code=DSS.WANT_CHANGE_DOU) &
        Q(
            before_desired_date_in_current_learn_year() |
            Q(desired_date__isnull=True)
        ) &
        with_health_need_filter() &
        Q(children__date_of_birth__lt=DateMixin._get_current_calendar_year())
    )


def get_203_filter(context):
    """Фильтр для показателя 20.3
    :return: Q
    """

    return (
        Q(status__code=DSS.WANT_CHANGE_DOU) &
        desired_date_interval_current_learn_year()
    )


def get_204_filter(context):
    """Фильтр для показателя 20.4
    :return: Q
    """

    return (
        Q(status__code=DSS.WANT_CHANGE_DOU) &
        Q(after_desired_date_in_next_learn_year())
    )


def get_30_filter(context):
    """Фильтр для показателя 30
    :return: Q
    """

    return Q(
        direct__status__code__in=[
            DRS.REGISTER, DRS.DOGOVOR
        ]
    ) & ~Q(direct__group__status=GroupStatusEnum.ARCHIVE)


def get_302_filter(context):
    """Фильтр для показателя 30.2
    :return: Q
    """
    return (
        Q(direct__status__code__in=[
            DRS.REGISTER, DRS.DOGOVOR]) & ~Q(
            direct__group__status=GroupStatusEnum.ARCHIVE)
    )


def get_33_filter(context):
    """Фильтр для показателя 33
    :return: Q
    """
    return (
        Q(status__code__in=[DSS.REFUSED, DSS.DIDNT_COME])
    )


def queue_register_with_direct(context):
    """Фильтр. Заявка в статусе "Зарегистрировано"
    или заявки в других статусах с направлениями в статусах
    "Подтверждено", "Предложено системой"
    """

    return Q(status__code=DSS.REGISTERED) | Q(
        Q(direct__status__code__in=[
            DRS.CONFIRM, DRS.NEW]) & ~Q(
            direct__group__status=GroupStatusEnum.ARCHIVE
        ))


def queue_with_direct(context):
    """Фильтр. Заявка в статусе "Зарегистрировано" или "Подтверждение льгот"
    или заявки в других статусах с направлениями в статусах
    "Заключение договора", "Подтверждено",
    "Направлен в ДОУ", "Предложено системой"
    """

    return Q(
        Q(status__code=DSS.REGISTERED) |
        Q(status__code=DSS.PRIV_CONFIRMATING)
    ) | Q(
        Q(direct__status__code__in=[
            DRS.CONFIRM, DRS.DOGOVOR, DRS.REGISTER, DRS.NEW]) & ~Q(
                direct__group__status=GroupStatusEnum.ARCHIVE)
    )


def queue_with_direct_status_confirm_new(context):
    """Фильтр. Заявка в статусе "Зарегистрировано" или "Подтверждение льгот"
    или заявки в других статусах с направлениями в статусах
    "Подтверждено", "Предложено системой"
    """

    return Q(
        Q(status__code=DSS.REGISTERED) |
        Q(status__code=DSS.PRIV_CONFIRMATING)) | Q(
            direct__status__code__in=[
                DRS.CONFIRM, DRS.NEW])


def status_registration_or_privilege_confirmation():
    """Фильтр Статус Зарегистрировано или ПОдтверждение льгот
    :return: Q
    """

    return Q(
        status__code=DSS.REGISTERED) | Q(status__code=DSS.PRIV_CONFIRMATING)


def queue_with_want_change_dou(context):
    """Фильтр. Заявка в статусе "Зарегистрировано" или "Подтверждение льгот"
    или "Желает изменить ДОУ"
    или заявки в других статусах с направлениями в статусах
    "Заключение договора", "Подтверждено",
    "Направлен в ДОУ", "Предложено системой"
    :param context:
    :return:
    """

    return Q(
        Q(status__code=DSS.REGISTERED) |
        Q(status__code=DSS.PRIV_CONFIRMATING) |
        Q(status__code=DSS.WANT_CHANGE_DOU)
    ) | Q(
        Q(
            direct__status__code__in=[
                DRS.CONFIRM, DRS.DOGOVOR, DRS.REGISTER, DRS.NEW]
        ) & ~Q(direct__group__status=GroupStatusEnum.ARCHIVE)
    )


def status_directed():
    """Фильтр Направлен в ДОУ
    :return: Q
    """

    return Q(status__code=DSS.DIRECTED)


def direct_status_registered_or_dogovor(context):
    """Направление в статусе Направлен в ДОУ или Заключение договора
    :return: Q
    """

    declarations_id_with_needed_directs = Direct.objects.filter(
        group__unit_id=context.unit.id,
        status__code__in=[
            DRS.REGISTER, DRS.DOGOVOR]).values_list(
                "declaration_id", flat=True).distinct()

    return Q(id__in=declarations_id_with_needed_directs)


def direct_in_common_group(context):
    """Фильтр Направления выданные в общеразвивающие группы в статусе
    Направлен в ДОУ или Заключение договора.
    Повторяет функциональность direct_status_registered_or_dogovor
    для того, чтобы не выполнять два запроса
    :return: Q
    """

    declarations_id_with_needed_directs = Direct.objects.filter(
        group__unit_id=context.unit.id,
        status__code__in=[DRS.REGISTER, DRS.DOGOVOR],
        group__type__code=GroupTypeEnumerate.DEV).values_list(
            "declaration_id", flat=True).distinct()

    return Q(id__in=declarations_id_with_needed_directs)


def desired_date_in_next_learn_year():
    """Фильтр дата желаемого зачисления >= 2/09/Следующего учебного года
    :return: Q
    """

    next_learn_year_begin, _ = DateMixin.get_next_learn_year()
    return Q(desired_date__gte=next_learn_year_begin)


def desired_date_interval_current_learn_year():
    """Фильтр дата желаемого зачисления > 1/09/Текущий учебный год и
     < 1/09/Следующего учебного года
    :return: Q
    """

    date_start = DateMixin._get_current_calendar_year()
    date_end = DateMixin._get_next_calendar_year()

    return Q(desired_date__gt=date_start) & Q(desired_date__lt=date_end)


def before_desired_date_in_current_learn_year():
    """Фильтр дата желаемого зачисления <= 1/09/Текущий учебный год
    :return: Q
    """

    date_border = DateMixin._get_current_calendar_year()

    return Q(desired_date__lte=date_border)


def after_desired_date_in_current_learn_year():
    """Фильтр дата желаемого зачисления > 1/09/Текущий учебный год
    :return: Q
    """

    date_border = DateMixin._get_current_calendar_year()

    return Q(desired_date__gt=date_border)


def after_desired_date_in_next_learn_year():
    """Фильтр дата желаемого зачисления > 1/09/Следующего учебного года
    :return: Q
    """

    date_border = DateMixin._get_next_calendar_year()

    return Q(desired_date__gte=date_border)


def before_desired_date_in_next_learn_year():
    """Фильтр дата желаемого зачисления <= 1/09/Следующего учебного года."""
    date_border = DateMixin._get_next_calendar_year()

    return Q(desired_date__lte=date_border)


def health_need_filter(health_need):
    """Фильтр потребностей по здоровью
    :return: Q
    """
    return Q(children__health_need_id__in=health_need)


def with_health_need_filter():
    """в заявлении в поле "Ограниченные возможности здоровья"
     значение не пусто и не равно "нет"
     Если фильтровать по коду,
     то формируется неверный запрос, пришлось зашить Id
    :return: Q
    """
    hn_ids = HealthNeed.objects.exclude(
        code=HealthNeed.NO).values_list('id', flat=True).distinct()
    return Q(children__health_need__id__in=hn_ids)


def without_pupil(context):
    """У ребенка в заявление нет зачисления в фактическую группу
    :return:
    """

    children_ids = Pupil.objects.filter(
        grup__status=GroupStatusEnum.FACT
    ).values_list(
        'children', flat=True).distinct()
    return ~Q(children__in=children_ids)
